/**************************************************************************//**
 * @file     main.c
 * @version  V2.00
 * $Revision: 4 $
 * $Date: 19/11/11 5:51p $
 * @brief    To show how to use the gpio to control LCD1602.
 *
 * @note
 * Copyright (C) 2019 Nuvoton Technology Corp. All rights reserved.
 ******************************************************************************/
#include <stdio.h>
#include "NUC230_240.h"

/*------------------------------------------------------------------------------------*/
/* Function Description                                                               */
/*------------------------------------------------------------------------------------*/

// GPIOE.0 -> LCD RS(Register Select)
//                RS = 0 : Cmd register
//                RS = 1 : Data register
// GPIOE.1 -> LCD R/W(Read/Write)
//                R/W = 0 : Write
//                R/W = 1 : Read
// GPIOE.2 -> LCD E(Enable)
//                E = 0 : LCD disable
//                E = 1 : LCD enable
// GPIOA.0 -> LCD DB0(Data Bus)
// GPIOA.1 -> LCD DB1
// GPIOA.2 -> LCD DB2
// GPIOA.3 -> LCD DB3
// GPIOA.4 -> LCD DB4
// GPIOA.5 -> LCD DB5
// GPIOA.6 -> LCD DB6
// GPIOA.7 -> LCD DB7

/*------------------------------------------------------------------------------------*/
/* Define                                                                             */
/*------------------------------------------------------------------------------------*/

#define LCD_DATA    PA->DOUT // GPIOA data out
#define LCD_RS_SET  PB0=1 // Data register
#define LCD_RS_CLR  PB0=0 // Cmd register
#define LCD_RW_SET  PB1=1 // Write
#define LCD_RW_CLR  PB1=0 // Read
#define LCD_E_SET   PB2=1 // LCD enable
#define LCD_E_CLR   PB2=0 // LCD disable

/*------------------------------------------------------------------------------------*/
/* Function                                                                           */
/*------------------------------------------------------------------------------------*/
void WriteDataLcd(unsigned char wdata)
{
    LCD_DATA = wdata;       // Set data out
    LCD_RS_SET;             // Select Data register
    LCD_RW_CLR;             // LCD1602 Read
    LCD_E_CLR;              // LCD disable
    CLK_SysTickDelay(1000); // Delay 1000 us

    LCD_E_SET;              // LCD enable
}

void WriteCommandLcd(unsigned char wdata)
{
    LCD_DATA = wdata;       // Set data out
    LCD_RS_CLR;             // Select Cmd register
    LCD_RW_CLR;             // LCD1602 Read
    LCD_E_CLR;              // LCD disable
    CLK_SysTickDelay(1000); // Delay 1000 us
    
    LCD_E_SET;              // LCD enable
}

void lcd_init(void)
{
    LCD_DATA = 0;
    WriteCommandLcd(0x38);  //DL=8-bit, N=2-line, F=5x8 dots
    CLK_SysTickDelay(50000);// Delay 50000 us
    WriteCommandLcd(0x08);  //D=0 Display Off, C=0 Cursor Off, B=0 Blinks Off.
    WriteCommandLcd(0x06);  //I/D=1 Increment, S=0 Display does not shift.
    WriteCommandLcd(0x0C);  //D=0 Display On, C=0 Cursor Off, B=0 Blinks Off.
}

void display_xy(unsigned char x, unsigned char y)
{
    x += 0x80;              // For 1st line
    if (y == 1)
        x += 0x40;          // For 2nd line
    WriteCommandLcd(x);     // Write LCD Command
}

void display_char(unsigned char x, unsigned char y, unsigned char dat)
{
    display_xy(x, y);  // Select line 1 or line 2
    WriteDataLcd(dat); // Write the data to LCD
}

void  display_string(unsigned  char x, unsigned  char y, unsigned  char *string)
{
    display_xy(x, y);           // Select line 1 or line 2

    while (*string)             // If there is not end of string, sending data continuously.
    {
        WriteDataLcd(*string);  // Write data to LCD
        string++;               // Move the pointer to next character
    }
}

unsigned char  LcdBuf1[] = {"Welcome!"};    // Declare a string "Welcome!"
unsigned char  LcdBuf2[] = {"NuMicro M0"};  // Declare a string "NuMicro M0" 

int main(void)
{
    GPIO_SetMode(PA, 0xFFFF, GPIO_PMD_OUTPUT);  // Set PA0 ~ PA15 to output mode.
    lcd_init();                                 // Initial LED module.
    display_string(2, 0, LcdBuf1);              // Display string 1 "Welcome!"
    display_string(0, 1, LcdBuf2);              // Display string 2 "NuMicro M0"

    while (1);
}
/*** (C) COPYRIGHT 2019 Nuvoton Technology Corp. ***/
